﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Diagnostics;
using System.Runtime.InteropServices;

namespace BIXOLON_SamplePg
{
    public partial class frmMain : Form
    {
        private BXLAPI.BxlCallBackDelegate statusCallBackDelegate = null;

        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            InitControls();
            EnableControls(false);
            EnableIFControls(BXLAPI.IUsb);

            lblNote.Text = "Note:" +  Environment.NewLine;
            lblNote.Text += "Display APIs work with " + Environment.NewLine;
            lblNote.Text += "SRP-Q300/Q302";
        }

        private void InitControls()
        {
            try{
                rdoIF_Usb.Checked               = true;
                cmbLPT_Port.SelectedIndex       = 0;
                cmbSerial_Port.SelectedIndex    = 0;
                cmbSerial_Baudrate.SelectedItem = "115200";
                cmbSerial_Databits.SelectedItem = "8";
                cmbSerial_Parity.SelectedItem   = "NONE";
                cmbSerial_Stopbits.SelectedItem = "1";
                cmbSerial_FlowControl.SelectedItem = "DTR/DSR";
            }
            catch (System.Exception ex){
                MessageBox.Show(ex.ToString());
            }
        }

        private void EnableControls(bool bConnected)
        {
            btnConnect.Enabled      = !bConnected;
            btnDisconnect.Enabled   = bConnected;
            btnFeedCut.Enabled      = bConnected;
            btnPrintText.Enabled    = bConnected;
            btnPrintBitmap.Enabled  = bConnected;
            btn1DBarcode.Enabled    = bConnected;
            btn2DBarcode.Enabled    = bConnected;
            btnDirectIO.Enabled     = bConnected;
            btnStatus.Enabled       = bConnected;
            btnPresenterStatus.Enabled = bConnected;
            btnPrintSample.Enabled  = bConnected;
            btnStatus_Set.Enabled   = bConnected;
            btnStatus_Cancel.Enabled = false;

            btnClearScreen.Enabled  = bConnected;
            btnTextDisplay.Enabled  = bConnected;
            btnStoreImg.Enabled     = bConnected;
            btnDisplayImg.Enabled   = bConnected;
            btnPmDirection.Enabled  = bConnected;
            btnPmSample.Enabled     = bConnected;
        }

        private void EnableIFControls(int nInterface)
        {
            cmbLPT_Port.Enabled         = nInterface == BXLAPI.IParallel;
            cmbSerial_Port.Enabled      = nInterface == BXLAPI.ISerial || nInterface == BXLAPI.IBluetooth;
            cmbSerial_Baudrate.Enabled  = nInterface == BXLAPI.ISerial;
            cmbSerial_Databits.Enabled  = nInterface == BXLAPI.ISerial;
            cmbSerial_Parity.Enabled    = nInterface == BXLAPI.ISerial;
            cmbSerial_Stopbits.Enabled  = nInterface == BXLAPI.ISerial;
            cmbSerial_FlowControl.Enabled = nInterface == BXLAPI.ISerial;
            txtNet_IPAddr.Enabled       = nInterface == BXLAPI.ILan || nInterface == BXLAPI.IWLan;
            txtNet_PortNum.Enabled      = nInterface == BXLAPI.ILan || nInterface == BXLAPI.IWLan;
        }

        private void rdoIF_Serial_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(BXLAPI.ISerial);
        }

        private void rdoIF_Parallel_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(BXLAPI.IParallel);
        }

        private void rdoIF_Usb_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(BXLAPI.IUsb);
        }

        private void rdoIF_Lan_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(BXLAPI.ILan);
        }

        private void rdoIF_WLan_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(BXLAPI.IWLan);
        }

        private void rdoIF_BT_CheckedChanged(object sender, EventArgs e)
        {
            EnableIFControls(BXLAPI.IBluetooth);
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            string strPort = "";
            int nBaudrate   = 115200, nDatabits = 8, nParity = 0, nStopbits = 0, nNet_PortNum = 0, nFlowControl = 0;

            if (rdoIF_Serial.Checked)
            {
                strPort     = cmbSerial_Port.Text;
                nBaudrate   = Convert.ToInt32(cmbSerial_Baudrate.Text);
                nDatabits   = Convert.ToInt32(cmbSerial_Databits.Text);
                nParity     = cmbSerial_Parity.SelectedIndex;
                nStopbits   = cmbSerial_Stopbits.SelectedIndex;
                nFlowControl = cmbSerial_FlowControl.SelectedIndex;

                if (BXLAPI.PrinterOpen(BXLAPI.ISerial, strPort, nBaudrate, nDatabits, nParity, nStopbits, nFlowControl) != BXLAPI.BXL_SUCCESS)
                {
                    MessageBox.Show("Connect fail [Serial]");
                    return;
                }
            }
            else if(rdoIF_Parallel.Checked)
            {
                strPort = cmbLPT_Port.Text;

                if (BXLAPI.PrinterOpen(BXLAPI.IParallel, strPort, 0, 0, 0, 0, 0) != BXLAPI.BXL_SUCCESS)
                {
                    MessageBox.Show("Connect fail [Parallel]");
                    return;
                }
            }
            else if(rdoIF_Usb.Checked)
            {
                if ( BXLAPI.PrinterOpen(BXLAPI.IUsb, "", 0, 0, 0, 0, 0) != BXLAPI.BXL_SUCCESS)
                {
                    MessageBox.Show("Connect fail [USB]");
                    return;
                }
            }
            else if (rdoIF_Lan.Checked || rdoIF_WLan.Checked)
            {
                strPort = txtNet_IPAddr.Text;
                nNet_PortNum = Convert.ToInt32(txtNet_PortNum.Text);

                if (BXLAPI.PrinterOpen(rdoIF_Lan.Checked ? BXLAPI.ILan : BXLAPI.IWLan, strPort, nNet_PortNum, 0, 0, 0, 0) != BXLAPI.BXL_SUCCESS)
                {
                    MessageBox.Show("Connect fail [TCP/IP]");
                    return;
                }
            }
            else if (rdoIF_BT.Checked)
            {
                strPort = cmbSerial_Port.Text;

                if (BXLAPI.PrinterOpen(BXLAPI.IBluetooth, strPort, 115200, 0, 0, 0, 0) != BXLAPI.BXL_SUCCESS)
                {
                    MessageBox.Show("Connect fail [Bluetooth]");
                    return;
                }
            }

            EnableControls(true);

            Cursor.Current = Cursors.Default;
        }

        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            BXLAPI.PrinterClose();
            EnableControls(false);
            statusTextBox.Clear();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            BXLAPI.PrinterClose();
        }

        private void btnFeedCut_Click(object sender, EventArgs e)
        {
            BXLAPI.CutPaper();
            //Only BK3-31 with presenter supported
            BXLAPI.PaperEject(BXLAPI.BXL_EJT_HOLD);
        }

        private void btnPrintText_Click(object sender, EventArgs e)
        {
            // Enters 'Transaction' mode.
            BXLAPI.TransactionStart();

            BXLAPI.InitializePrinter();
            BXLAPI.SetCharacterSet(BXLAPI.BXL_CS_WPC1252);
            BXLAPI.SetInterChrSet(BXLAPI.BXL_ICS_USA);

            // Call 'PrintText' with system default code page
            BXLAPI.PrintText("Left, Default\n", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.PrintText("Center, Underline\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_UNDERLINE, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.PrintText("Left, Underline, Bold\n", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_UNDERLINE | BXLAPI.BXL_FT_BOLD, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.PrintText("Left, Underline, Bold, FontB\n",BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_UNDERLINE | BXLAPI.BXL_FT_BOLD | BXLAPI.BXL_FT_FONTB, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.PrintText("Left, Underline, Bold, FontC\n",BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_UNDERLINE | BXLAPI.BXL_FT_BOLD | BXLAPI.BXL_FT_FONTC, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.PrintText("Right, Reverse\n",BXLAPI.BXL_ALIGNMENT_RIGHT, BXLAPI.BXL_FT_REVERSE, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.PrintText("Default, 0W 0H\n",BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.PrintText("Double Width\n", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_1WIDTH);
            BXLAPI.PrintText("Double Height\n",BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_1HEIGHT);
            //BXLAPI.PrintText("Color in RED\n", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_RED_COLOR, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            
            //BXLAPI.PrintTextW("Xin chào\n\n", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT, BXLAPI.BXL_CS_TCVN);		// Vietnamese
            //BXLAPI.PrintTextW("안녕하세요\n\n", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT, BXLAPI.BXL_CS_KS5601);	// Korean
            //BXLAPI.PrintTextW("你好\n\n", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT, BXLAPI.BXL_CS_BIG5);			// Chinese (BIG5)
            //BXLAPI.PrintTextW("你好\n\n", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT, BXLAPI.BXL_CS_GB2312);			// Chinese (gb2312)
            //BXLAPI.PrintTextW("こんにちは\n\n", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT, BXLAPI.BXL_CS_SHIFT_JIS);	// Shift_JIS

            BXLAPI.CutPaper();

            //Only BK3-31 with presenter supported
            BXLAPI.PaperEject(BXLAPI.BXL_EJT_HOLD);

            // Leaves 'Transaction' mode, and then sends print data in the buffer to start printing.
            if (BXLAPI.TransactionEnd(true, 3000 /* 3 seconds */) != BXLAPI.BXL_SUCCESS){
                // failed to read a response from the printer after sending the print-data.
                MessageBox.Show("TransactionEnd failed.");
            }
        }

        private void btnPrintBitmap_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileOpen     = new OpenFileDialog();
            fileOpen.DefaultExt         = "";
            fileOpen.InitialDirectory   = "";
            fileOpen.Filter             = "Image File(*.bmp)|*.bmp";
            fileOpen.FilterIndex        = 1;
            fileOpen.RestoreDirectory   = true;

            if (fileOpen.ShowDialog() != DialogResult.OK)
                return;

            // Enters 'Transaction' mode.
            BXLAPI.TransactionStart();

            // #1. Print a graphic image using "PrintBitmap" function. 
            BXLAPI.PrintBitmap(fileOpen.FileName, BXLAPI.BXL_WIDTH_NONE, BXLAPI.BXL_ALIGNMENT_CENTER, 50, true);

            // #2. Print a graphic image using "GetPrintBitmapBuf" function. 
            //The GetPrintBitmapBuf function retrieves data for the specified graphic file.
            /*
            byte[] pOutBuffer   = null;
            uint cbNeeded       = 0;
            long lResult        = 0;

            lResult     = BXLAPI.GetBitmapBuffer(fileOpen.FileName, BXLAPI.BXL_WIDTH_NONE, 50, true, pOutBuffer, cbNeeded, ref cbNeeded);
            pOutBuffer  = new byte[cbNeeded];
            lResult     = BXLAPI.GetBitmapBuffer(fileOpen.FileName, BXLAPI.BXL_WIDTH_NONE, 50, true, pOutBuffer, cbNeeded, ref cbNeeded);

            if (lResult == BXLAPI.BXL_SUCCESS) {
                int written = 0;
                BXLAPI.WriteBuff(pOutBuffer, (int)cbNeeded, ref written);
            }
            */

            //BXLAPI.LineFeed(2);
            BXLAPI.CutPaper();

            //Only BK3-31 with presenter supported
            BXLAPI.PaperEject(BXLAPI.BXL_EJT_HOLD);

            // Leaves 'Transaction' mode, and then sends print data in the buffer to start printing.
            if (BXLAPI.TransactionEnd(true, 3000 /* 3 seconds */) != BXLAPI.BXL_SUCCESS){
                // failed to read a response from the printer after sending the print-data.
                MessageBox.Show("TransactionEnd failed.");
            }
        }

        private void btn1DBarcode_Click(object sender, EventArgs e)
        {
            // Enters 'Transaction' mode.
            BXLAPI.TransactionStart();

            //UPC-A
            BXLAPI.PrintText("BARCODE : UPCA\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintBarcode("012345678912", BXLAPI.BXL_BCS_UPCA, 50, 2, BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_BC_TEXT_BELOW);
            BXLAPI.LineFeed(2);

            //UPC-E
            BXLAPI.PrintText("BARCODE : UPCE\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintBarcode("012345678912", BXLAPI.BXL_BCS_UPCE, 50, 2, BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_BC_TEXT_BELOW);
            BXLAPI.LineFeed(2);

            //JAN,EAN8	
            BXLAPI.PrintText("BARCODE : JAN/EAN8\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintBarcode("01234567", BXLAPI.BXL_BCS_JAN8, 50, 2, BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_BC_TEXT_BELOW);
            BXLAPI.LineFeed(2);


            //JAN,EAN13	
            BXLAPI.PrintText("BARCODE : JAN/EAN13\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintBarcode("012345678912", BXLAPI.BXL_BCS_EAN13, 50, 2, BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_BC_TEXT_BELOW);
            BXLAPI.LineFeed(2);

            // Code39
            BXLAPI.PrintText("BARCODE : CODE39\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintBarcode("0123456789", BXLAPI.BXL_BCS_CODE39, 50, 2, BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_BC_TEXT_BELOW);
            BXLAPI.LineFeed(2);

            // ITF
            BXLAPI.PrintText("BARCODE : ITF\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintBarcode("0123456789", BXLAPI.BXL_BCS_ITF, 50, 2, BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_BC_TEXT_BELOW);
            BXLAPI.LineFeed(2);

            // CODABAR
            BXLAPI.PrintText("BARCODE : CODABAR\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintBarcode("A0123456789", BXLAPI.BXL_BCS_CODABAR, 50, 2, BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_BC_TEXT_BELOW);
            BXLAPI.LineFeed(2);

            // CODE3
            BXLAPI.PrintText("BARCODE : CODE93\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintBarcode("0123456789", BXLAPI.BXL_BCS_CODE93, 50, 2, BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_BC_TEXT_BELOW);
            BXLAPI.LineFeed(2);

            // CODE128
            BXLAPI.PrintText("BARCODE : CODE128\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintBarcode("0123456789", BXLAPI.BXL_BCS_CODE128, 50, 2, BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_BC_TEXT_BELOW);

            BXLAPI.CutPaper();

            //Only BK3-31 with presenter supported
            BXLAPI.PaperEject(BXLAPI.BXL_EJT_HOLD);

            // Leaves 'Transaction' mode, and then sends print data in the buffer to start printing.
            if (BXLAPI.TransactionEnd(true, 3000 /* 3 seconds */) != BXLAPI.BXL_SUCCESS){
                // failed to read a response from the printer after sending the print-data.
                MessageBox.Show("TransactionEnd failed.");
            }
        }

        private void btn2DBarcode_Click(object sender, EventArgs e)
        {
            // Enters 'Transaction' mode.
            BXLAPI.TransactionStart();

            BXLAPI.PrintText("PDF417\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintPDF417("1234567890ABCDEFG", BXLAPI.BXL_PDF417_TYPE1, 0, 0, 3, 2, BXLAPI.BXL_PDF417_ECC_LEVEL_0, BXLAPI.BXL_ALIGNMENT_LEFT);
            BXLAPI.LineFeed(2);

            BXLAPI.PrintText("QRCode\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintQRCode("1234567890ABCDEFG", BXLAPI.BXL_QRCODE_MODEL1, 4, BXLAPI.BXL_QRCODE_ECC_LEVEL_L, BXLAPI.BXL_ALIGNMENT_LEFT);
            BXLAPI.LineFeed(2);

            BXLAPI.PrintText("DataMatrix\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintDataMatrix("1234567890ABCDEFG", 3, BXLAPI.BXL_ALIGNMENT_LEFT);
            BXLAPI.LineFeed(2);
            
            /*
            BXLAPI.PrintText("GS1DataBar\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintGS1DataBar("1234567890ABCDEFG", BXLAPI.BXL_GS1DATABAR_STACKED, 3, BXLAPI.BXL_ALIGNMENT_LEFT);
            BXLAPI.LineFeed(2);

            BXLAPI.PrintText("CompositeSymbology\n", BXLAPI.BXL_ALIGNMENT_LEFT, 0, 0);
            BXLAPI.PrintCompositeSymbology("1234567890", BXLAPI.BXL_COMPOSITE_LINE_EAN8,
                                            "ABCDEFGHIJ", BXLAPI.BXL_COMPOSITE_2D_AUTO, 
                                            2, BXLAPI.BXL_ALIGNMENT_LEFT);
            */
            BXLAPI.CutPaper();

            //Only BK3-31 with presenter supported
            BXLAPI.PaperEject(BXLAPI.BXL_EJT_HOLD);

            // Leaves 'Transaction' mode, and then sends print data in the buffer to start printing.
            if (BXLAPI.TransactionEnd(true, 3000 /* 3 seconds */) != BXLAPI.BXL_SUCCESS){
                // failed to read a response from the printer after sending the print-data.
                MessageBox.Show("TransactionEnd failed.");
            }
        }

        private void btnDirectIO_Click(object sender, EventArgs e)
        {
            string strEr;
            string strRet;

            Byte dbRead = 0x00;
	        uint dwRead = 1;

            int lResult = BXLAPI.DirectIO(new byte[] { 0x10, 0x04, 0x02 }, 3, ref dbRead, ref dwRead);

            if (lResult != BXLAPI.BXL_SUCCESS)
	        {
		        strEr = "Error Code = " + lResult;
		        MessageBox.Show(strEr);		
		        return;
	        }
	        else{
                strRet = String.Format("Read Data : 0x{0:X}", dbRead);
		        MessageBox.Show(strRet);
	        }
        }

        private void btnStatus_Click(object sender, EventArgs e)
        {
            string strStatus = "Printer Status = ";
            int lState = (int)BXLAPI.GetPrinterCurrentStatus();

            if (lState == BXLAPI.BXL_STS_NORMAL){
                strStatus += " NORMAL ";
                MessageBox.Show(strStatus);
                return;
            }
            if ((lState & BXLAPI.BXL_STS_PAPER_NEAR_END) == BXLAPI.BXL_STS_PAPER_NEAR_END)          strStatus += " PAPER NEAR END ";
            if ((lState & BXLAPI.BXL_STS_PAPEREMPTY) == BXLAPI.BXL_STS_PAPEREMPTY)                  strStatus += " PAPER EMPTY ";
            if ((lState & BXLAPI.BXL_STS_CASHDRAWER_HIGH) == BXLAPI.BXL_STS_CASHDRAWER_HIGH)        strStatus += " CASHDRAWER HIGH ";
            if ((lState & BXLAPI.BXL_STS_CASHDRAWER_LOW) == BXLAPI.BXL_STS_CASHDRAWER_LOW)          strStatus += " CASHDRAWER LOW ";
            if ((lState & BXLAPI.BXL_STS_COVEROPEN) == BXLAPI.BXL_STS_COVEROPEN)                    strStatus += " COVER OPEN ";
            if ((lState & BXLAPI.BXL_STS_BATTERY_LOW) == BXLAPI.BXL_STS_BATTERY_LOW)                strStatus += " BATTERY LOW ";
            if ((lState & BXLAPI.BXL_STS_PAPER_TO_BE_TAKEN) == BXLAPI.BXL_STS_PAPER_TO_BE_TAKEN)    strStatus += " PAPER PRESENCE ";
            if ((lState & BXLAPI.BXL_STS_ERROR) == BXLAPI.BXL_STS_ERROR){
                // OFFLINE
                strStatus += " OFFLINE ";
            }

            MessageBox.Show(strStatus);
        }

        private void btnPrintSample_Click(object sender, EventArgs e)
        {
            // Enters 'Transaction' mode.
            BXLAPI.TransactionStart();

            BXLAPI.InitializePrinter();
            BXLAPI.SetCharacterSet(BXLAPI.BXL_CS_WPC1252);
            BXLAPI.SetInterChrSet(BXLAPI.BXL_ICS_USA);

	        BXLAPI.PrintText("Receipt", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_BOLD | BXLAPI.BXL_FT_UNDERLINE, BXLAPI.BXL_TS_1WIDTH | BXLAPI.BXL_TS_1HEIGHT);	
	        BXLAPI.PrintText(" \n\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_BOLD | BXLAPI.BXL_FT_UNDERLINE, BXLAPI.BXL_TS_1WIDTH | BXLAPI.BXL_TS_1HEIGHT);	

	        BXLAPI.PrintText("------------------------------\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
	        BXLAPI.PrintText(" Apple      3             $3.0\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
	        BXLAPI.PrintText(" Cola       1             $1.2\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	
	        BXLAPI.PrintText(" Milk       2             $1.8\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	
	        BXLAPI.PrintText(" Juice      1             $2.0\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	
	        BXLAPI.PrintText("------------------------------\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	
	        BXLAPI.PrintText(" Total                    $8.0\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_BOLD, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	
	        BXLAPI.PrintText(" Cash                     $8.0\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_BOLD, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	
	        BXLAPI.PrintText("                          $0.0\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_BOLD, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	

	        BXLAPI.PrintText("Test", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_BOLD, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	
	        BXLAPI.PrintText("BBB", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_FONTB | BXLAPI.BXL_FT_BOLD, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	
	        BXLAPI.PrintText("CCC", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_FONTB | BXLAPI.BXL_FT_REVERSE, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	
	        BXLAPI.PrintText("DDD", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_BOLD | BXLAPI.BXL_FT_UNDERLINE, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);	
	        BXLAPI.PrintText("\n", BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_FT_BOLD | BXLAPI.BXL_FT_UNDERLINE, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.PrintBarcode("12345678", BXLAPI.BXL_BCS_ITF, 100, 2, BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_BC_TEXT_BELOW);

            BXLAPI.OpenDrawer(BXLAPI.BXL_CASHDRAWER_PIN2);
            //BXLAPI.OpenDrawer(BXLAPI.BXL_CASHDRAWER_PIN5);
            BXLAPI.CutPaper();

            //Only BK3-31 with presenter supported
            BXLAPI.PaperEject(BXLAPI.BXL_EJT_HOLD);

            // Leaves 'Transaction' mode, and then sends print data in the buffer to start printing.
            if (BXLAPI.TransactionEnd(true, 3000 /* 3 seconds */) != BXLAPI.BXL_SUCCESS){
                // failed to read a response from the printer after sending the print-data.
                MessageBox.Show("TransactionEnd failed.");
            }
        }

        private void btnClearScreen_Click(object sender, EventArgs e)
        {
            BXLAPI.ClearScreen();         // Clear screen (all lines)            
            //BXLAPI.ClearLine(1);        // Clear 1st line
            //BXLAPI.ClearLine(2);        // Clear 2nd line
        }

        private void btnTextDisplay_Click(object sender, EventArgs e)
        {
            BXLAPI.SetInternationalCharacterset(BXLAPI.BXL_ICS_USA);
            BXLAPI.DisplayString("TEST ");
            BXLAPI.DisplayStringW("TEST ", BXLAPI.BXL_TEXT_ENCODE_PC437);

            //BXLAPI.DisplayStringAtLine(1, "TEST ");
            //BXLAPI.DisplayStringAtLine(2, "TEST ");
            //BXLAPI.DisplayStringAtLineW(1, "TEST ", BXLAPI.BXL_TEXT_ENCODE_PC437);
            //BXLAPI.DisplayStringAtLineW(2, "TEST ", BXLAPI.BXL_TEXT_ENCODE_PC437);
        }

        private void btnStoreImg_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileOpen     = new OpenFileDialog();
            fileOpen.DefaultExt         = "";
            fileOpen.InitialDirectory   = "";
            fileOpen.Filter             = "Image File(*.bmp)|*.bmp";
            fileOpen.FilterIndex        = 1;
            fileOpen.RestoreDirectory   = true;

            if (fileOpen.ShowDialog() != DialogResult.OK)
                return;

            BXLAPI.StoreImageFile(fileOpen.FileName, 1);

            // BCD-3000
            // 0 <= x Posistion <= 160, 
            // 0 <= y Posistion <= 32
            BXLAPI.DisplayImage(1, 0, 0);
        }

        private void btnDisplayImg_Click(object sender, EventArgs e)
        {
            BXLAPI.DisplayImage(1, 0, 0);
        }

        public int statusCallBackMethod(int nStatus)
        {
            return interpretPrinterStatusData(nStatus);
        }

        public int interpretPrinterStatusData(int status)
        {
            statusTextBox.AppendText(DateTime.Now + " : " + "\r\n");

            if (status == BXLAPI.BXL_STS_NORMAL){
                statusTextBox.AppendText(" - NORMAL" + "\r\n");
                return 0;
            }
            if ((status & BXLAPI.BXL_STS_PAPER_NEAR_END) == BXLAPI.BXL_STS_PAPER_NEAR_END)
                statusTextBox.AppendText(" - PAPER-NEAR-END" + "\r\n");
            if ((status & BXLAPI.BXL_STS_PAPEREMPTY) == BXLAPI.BXL_STS_PAPEREMPTY) 
                statusTextBox.AppendText(" - PAPER-EMPTY" + "\r\n");
            if ((status & BXLAPI.BXL_STS_CASHDRAWER_HIGH) == BXLAPI.BXL_STS_CASHDRAWER_HIGH)
                statusTextBox.AppendText(" - CASHDRAWER-HIGH" + "\r\n");
            if ((status & BXLAPI.BXL_STS_CASHDRAWER_LOW) == BXLAPI.BXL_STS_CASHDRAWER_LOW)
                statusTextBox.AppendText(" - CASHDRAWER-LOW" + "\r\n");
            if ((status & BXLAPI.BXL_STS_COVEROPEN) == BXLAPI.BXL_STS_COVEROPEN)
                statusTextBox.AppendText(" - COVER-OPEN" + "\r\n");
            if ((status & BXLAPI.BXL_STS_ERROR) == BXLAPI.BXL_STS_ERROR)
                statusTextBox.AppendText(" - OFFLINE" + "\r\n");
            if ((status & BXLAPI.BXL_STS_BATTERY_LOW) == BXLAPI.BXL_STS_BATTERY_LOW)
                statusTextBox.AppendText(" - BATTERY-LOW" + "\r\n");
            if ((status & BXLAPI.BXL_STS_PAPER_TO_BE_TAKEN) == BXLAPI.BXL_STS_PAPER_TO_BE_TAKEN)
                statusTextBox.AppendText(" - PAPER-PRESENCE" + "\r\n");
            return 0;
        }

        private void btnStatus_Set_Click(object sender, EventArgs e)
        {
            statusCallBackDelegate = new BXLAPI.BxlCallBackDelegate(statusCallBackMethod);
            if (BXLAPI.BidiSetCallBack(statusCallBackDelegate) != BXLAPI.BXL_SUCCESS){
                statusCallBackDelegate = null;
            }
            else{
                btnStatus_Set.Enabled = false;
                btnStatus_Cancel.Enabled = true;
            }

        }

        private void btnStatus_Cancel_Click(object sender, EventArgs e)
        {
            BXLAPI.BidiCancelCallBack();
            statusCallBackDelegate = null;
            btnStatus_Set.Enabled = true;
            btnStatus_Cancel.Enabled = false;
            statusTextBox.Clear();
        }

        private void btnPmDirection_Click(object sender, EventArgs e)
        {
            // Enters 'Transaction' mode.
            BXLAPI.TransactionStart();

            BXLAPI.InitializePrinter();
            BXLAPI.SetPagemode(BXLAPI.BXL_PRINTER_PAGEMODE_IN /* PAGE MODE IN */);
            BXLAPI.SetPagemodePrintArea(0, 0, 512, 600);
            BXLAPI.SetPagemodeDirection(BXLAPI.BXL_PRINTER_PD_0);
            BXLAPI.PrintText("Left to right", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.SetPagemodeDirection(BXLAPI.BXL_PRINTER_PD_LEFT90);
            BXLAPI.PrintText("Bottom to Top", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.SetPagemodeDirection(BXLAPI.BXL_PRINTER_PD_180);
            BXLAPI.PrintText("Right to Left", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.SetPagemodeDirection(BXLAPI.BXL_PRINTER_PD_RIGHT90);
            BXLAPI.PrintText("Top to Bottom", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            BXLAPI.SetPagemode(BXLAPI.BXL_PRINTER_PAGEMODE_OUT /* PAGE MODE OUT */);
            BXLAPI.CutPaper();

            //Only BK3-31 with presenter supported
            BXLAPI.PaperEject(BXLAPI.BXL_EJT_HOLD);

            // Leaves 'Transaction' mode, and then sends print data in the buffer to start printing.
            if (BXLAPI.TransactionEnd(true, 3000 /* 3 seconds */) != BXLAPI.BXL_SUCCESS){
                // failed to read a response from the printer after sending the print-data.
                MessageBox.Show("TransactionEnd failed.");
            }
        }

        private void btnPmSample_Click(object sender, EventArgs e)
        {
            // Enters 'Transaction' mode.
            BXLAPI.TransactionStart();

            int xPosition = 50;
            int yPosition = 50;
            BXLAPI.InitializePrinter();
            BXLAPI.SetPagemode(BXLAPI.BXL_PRINTER_PAGEMODE_IN /* PAGE MODE IN */);
            BXLAPI.SetPagemodePrintArea(0, 0, 512, 600);
            BXLAPI.SetPagemodeDirection(BXLAPI.BXL_PRINTER_PD_RIGHT90 /* TOP TO BOTTOM*/);
            BXLAPI.SetPagemodePosition(xPosition, yPosition);
            BXLAPI.PrintText("Sample Text", BXLAPI.BXL_ALIGNMENT_LEFT, BXLAPI.BXL_FT_DEFAULT, BXLAPI.BXL_TS_0WIDTH | BXLAPI.BXL_TS_0HEIGHT);
            yPosition += 70;
            BXLAPI.SetPagemodePosition(xPosition, yPosition);
            BXLAPI.PrintBarcode("12345678", BXLAPI.BXL_BCS_CODE128, 100, 2, BXLAPI.BXL_ALIGNMENT_CENTER, BXLAPI.BXL_BC_TEXT_NONE);
            BXLAPI.SetPagemode(BXLAPI.BXL_PRINTER_PAGEMODE_OUT /* PAGE MODE OUT */);
            BXLAPI.CutPaper();

            //Only BK3-31 with presenter supported
            BXLAPI.PaperEject(BXLAPI.BXL_EJT_HOLD);

            // Leaves 'Transaction' mode, and then sends print data in the buffer to start printing.
            if (BXLAPI.TransactionEnd(true, 3000 /* 3 seconds */) != BXLAPI.BXL_SUCCESS){
                // failed to read a response from the printer after sending the print-data.
                MessageBox.Show("TransactionEnd failed.");
            }
        }

        //Only BK3-31 with presenter supported
        private void btnPresenterStatus_Click(object sender, EventArgs e)
        {
            string strStatus = "Presenter Status = ";
            int lState = (int)BXLAPI.GetPresenterStatus();

            if (lState == BXLAPI.BXL_STS_NORMAL)
            {
                strStatus += " NORMAL ";
                MessageBox.Show(strStatus);
                return;
            }
            else if (lState == BXLAPI.BXL_NOT_SUPPORT)
            {
                strStatus = "Only BK3-31 with presenter supported";
                MessageBox.Show(strStatus);
                return;
            }
            else if (lState < BXLAPI.BXL_STS_NORMAL)
            {
                return;
            }

            if ((lState & BXLAPI.BXL_PRT_STS_PAPER_NEAR_END) == BXLAPI.BXL_PRT_STS_PAPER_NEAR_END) strStatus += " PAPER NEAR END ";
            if ((lState & BXLAPI.BXL_PRT_STS_PAPEREMPTY) == BXLAPI.BXL_PRT_STS_PAPEREMPTY) strStatus += " PAPER EMPTY ";
            if ((lState & BXLAPI.BXL_PRT_STS_PAPER_IN) == BXLAPI.BXL_PRT_STS_PAPER_IN) strStatus += " PAPER IN PRESENTER ";
            if ((lState & BXLAPI.BXL_PRT_STS_PAPER_JAM) == BXLAPI.BXL_PRT_STS_PAPER_JAM) strStatus += " PAPER JAM ";
            if ((lState & BXLAPI.BXL_STS_ERROR) == BXLAPI.BXL_STS_ERROR)
            {
                // OFFLINE
                strStatus += " OFFLINE ";
            }

            MessageBox.Show(strStatus);
        }
    }
}